@@ -7,7 +7,7 @@ import traceback
 from logging import Logger
 
 from functools import wraps
-from typing import Any, Union, Tuple, Dict, Optional, Type, List
+from typing import Any, Union, Tuple, Dict, Optional, Type, List, Callable, TypeVar, Generic
 
 from .. import NodeBase
 from .. import QtGui, QtCore, Signal, Slot, QtWidgets
@@ -18,10 +18,12 @@ __author__ = 'Wolfgang Pfaff'
 __license__ = 'MIT'
 
 
-# TODO: implement a threaded version of Node
+R = TypeVar('R', bound="Node")
+S = TypeVar('S')
+T = TypeVar('T')
 
 
-def updateOption(optName: Union[None, str] = None):
+def updateOption(optName: Optional[str] = None) -> Callable[[Callable[[R, S], T]], Callable[[R, S], T]]:
     """Decorator for property setters that are handy for user options.
 
     Property setters in nodes that are decorated with this will do two things:
@@ -31,9 +33,9 @@ def updateOption(optName: Union[None, str] = None):
     :param optName: name of the property.
     """
 
-    def decorator(func):
+    def decorator(func: Callable[[R, S], T]) -> Callable[[R, S], T]:
         @wraps(func)
-        def wrap(self, val):
+        def wrap(self: R, val: S) -> T:
             ret = func(self, val)
             if optName is not None and self.ui is not None and \
                     optName in self.ui.optSetters:
@@ -46,7 +48,11 @@ def updateOption(optName: Union[None, str] = None):
     return decorator
 
 
-def updateGuiFromNode(func):
+U = TypeVar('U', bound="NodeWidget")
+V = TypeVar('V',)
+
+
+def updateGuiFromNode(func: Callable[..., V]) -> Callable[..., V]:
     """
     Decorator for the UI to set an internal flag to during execution of
     the wrapped function. Prevents recursive updating (i.e., if
@@ -55,7 +61,7 @@ def updateGuiFromNode(func):
     """
 
     @wraps(func)
-    def wrap(self, *arg, **kw):
+    def wrap(self: U, *arg: Any, **kw: Any) -> V:
         self._emitGuiChange = False
         ret = func(self, *arg, **kw)
         self._emitGuiChange = True
@@ -66,8 +72,10 @@ def updateGuiFromNode(func):
 
 updateGuiQuietly = updateGuiFromNode
 
+W = TypeVar('W')
 
-def emitGuiUpdate(signalName: str):
+
+def emitGuiUpdate(signalName: str) -> Callable[[Callable[..., Any]], Callable[..., None]]:
     """
     Decorator for UI functions to emit the signal ``signalName``
     (given as argument the decorator), with the return of the wrapped function.
@@ -79,9 +87,9 @@ def emitGuiUpdate(signalName: str):
     :param signalName: name of the signal to emit.
     """
 
-    def decorator(func):
+    def decorator(func: Callable[..., Any]) -> Callable[..., None]:
         @wraps(func)
-        def wrap(self, *arg, **kw):
+        def wrap(self: W, *arg: Any, **kw: Any) -> None:
             ret = func(self, *arg, **kw)
             emit = getattr(self, '_emitGuiChange', True)
             if emit:
@@ -92,8 +100,11 @@ def emitGuiUpdate(signalName: str):
 
     return decorator
 
+NodeWidgetType = TypeVar("NodeWidgetType", bound="NodeWidget")
 
-class Node(NodeBase):
+# TODO: should we add a list of options to the class?
+#   that would allow programmatic syncing from node to widget, for instance.
+class Node(NodeBase, Generic[NodeWidgetType]):
     """Base class of the Node we use for plotter.
 
     This class inherits from ``pyqtgraph``'s Node, and adds a few additional
@@ -111,7 +122,7 @@ class Node(NodeBase):
 
     #: UI node widget class. If not None, and ``useUi`` is ``True``, an
     #: instance of the widget is created, and signal/slots are connected.
-    uiClass: Optional[Type["NodeWidget"]] = None
+    uiClass: Optional[Type[NodeWidgetType]] = None
 
     #: Whether or not to automatically set up a UI widget.
     useUi = True
@@ -127,6 +138,10 @@ class Node(NodeBase):
     #: emits a the list of names of new axes
     dataAxesChanged = Signal(list)
 
+    #: signal emitted when available dependents change
+    #: emits a the list of names of new dependents
+    dataDependentsChanged = Signal(list)
+
     #: signal emitted when any available data fields change (dep. and indep.)
     #: emits a the list of names of new axes
     dataFieldsChanged = Signal(list)
@@ -162,12 +177,12 @@ class Node(NodeBase):
         self.dataStructure: Optional[DataDictBase] = None
 
         if self.useUi and self.__class__.uiClass is not None:
-            self.ui: Optional["NodeWidget"] = self.__class__.uiClass(node=self)
+            self.ui: Optional["NodeWidgetType"] = self.__class__.uiClass(node=self)
             self.setupUi()
         else:
             self.ui = None
 
-    def setupUi(self):
+    def setupUi(self) -> None:
         """ setting up the UI widget.
 
         Gets called automatically in the node initialization.
@@ -177,6 +192,7 @@ class Node(NodeBase):
         UI widget (like connecting additional signals/slots between node and
         node widget).
         """
+        assert self.ui is not None
         self.ui.optionToNode.connect(self.setOption)
         self.ui.allOptionsToNode.connect(self.setOptions)
         self.optionChangeNotification.connect(self.ui.setOptionsFromNode)
@@ -186,7 +202,7 @@ class Node(NodeBase):
         """
         return self.ui
 
-    def setOption(self, nameAndVal: Tuple[str, Any]):
+    def setOption(self, nameAndVal: Tuple[str, Any]) -> None:
         """Set an option.
 
         name is the name of the property, not the string used for referencing
@@ -197,7 +213,7 @@ class Node(NodeBase):
         name, val = nameAndVal
         setattr(self, name, val)
 
-    def setOptions(self, opts: Dict[str, Any]):
+    def setOptions(self, opts: Dict[str, Any]) -> None:
         """Set multiple options.
 
         :param opts: a dictionary of property name : value pairs.
@@ -205,7 +221,7 @@ class Node(NodeBase):
         for opt, val in opts.items():
             setattr(self, opt, val)
 
-    def update(self, signal=True):
+    def update(self, signal: bool = True) -> None:
         super().update(signal=signal)
         if Node._raiseExceptions and self.exception is not None:
             raise self.exception[1]
@@ -221,13 +237,12 @@ class Node(NodeBase):
 
         :return: logger with a name that can be traced back easily to this node.
         """
-        name = self.__module__ + '.' + self.__class__.__name__ + '.' \
-               + self.name()
-        logger = log.getLogger()
+        name = f"{self.__module__}.{self.__class__.__name__}.{self.name()}"
+        logger = log.getLogger(name)
         logger.setLevel(log.LEVEL)
         return logger
 
-    def validateOptions(self, data: Any) -> bool:
+    def validateOptions(self, data: DataDictBase) -> bool:
         """Validate the user options
 
         Does nothing in this base implementation. Can be reimplemented by any
@@ -237,71 +252,79 @@ class Node(NodeBase):
         """
         return True
 
-    def process(self, dataIn: DataDictBase=None):
+    # TODO: should think about nodes with multiple inputs -- how would this look then?
+    # FIXME: return should only be Optional[Dict[str, DataDictBase]]
+    def process(self, dataIn: Optional[DataDictBase]=None) -> Optional[Dict[str, Optional[DataDictBase]]]:
         if dataIn is None:
             return None
 
-        if isinstance(dataIn, DataDictBase):
-            dtype = type(dataIn)
-            daxes = dataIn.axes()
-            ddeps = dataIn.dependents()
-            dshapes = dataIn.shapes()
+        if not isinstance(dataIn, DataDictBase):
+            raise ValueError('Unsupported data format provided.')
 
-            _axesChanged = False
-            _fieldsChanged = False
-            _typeChanged = False
-            _structChanged = False
-            _shapesChanged = False
+        _axesChanged = False
+        _fieldsChanged = False
+        _typeChanged = False
+        _structChanged = False
+        _shapesChanged = False
+        _depsChanged = False
 
+        dtype = type(dataIn)
+        daxes = dataIn.axes()
+        ddeps = dataIn.dependents()
+        dshapes = dataIn.shapes()
+        dstruct = dataIn.structure(add_shape=False)
+
+        if None in [self.dataAxes, self.dataDependents, self.dataType, self.dataShapes]:
+            _axesChanged = True
+            _fieldsChanged = True
+            _typeChanged = True
+            _structChanged = True
+            _shapesChanged = True
+            _depsChanged = True
+
+        else:
             if daxes != self.dataAxes:
+                _fieldsChanged = True
+                _structChanged = True
                 _axesChanged = True
 
-            if daxes != self.dataAxes or ddeps != self.dataDependents:
+            if ddeps != self.dataDependents:
                 _fieldsChanged = True
+                _structChanged = True
+                _depsChanged = True
 
             if dtype != self.dataType:
                 _typeChanged = True
-
-            if dtype != self.dataType or daxes != self.dataAxes \
-                    or ddeps != self.dataDependents:
                 _structChanged = True
 
             if dshapes != self.dataShapes:
                 _shapesChanged = True
 
-            self.dataAxes = daxes
-            self.dataDependents = ddeps
-            self.dataType = dtype
-            self.dataShapes = dshapes
-            self.dataStructure = dataIn.structure(add_shape=False)
+        self.dataAxes = daxes
+        self.dataDependents = ddeps
+        self.dataType = dtype
+        self.dataShapes = dshapes
+        self.dataStructure = dstruct
 
-            if _axesChanged:
-                self.dataAxesChanged.emit(daxes)
+        if _axesChanged:
+            self.dataAxesChanged.emit(daxes)
 
-            if _fieldsChanged:
-                self.dataFieldsChanged.emit(daxes + ddeps)
+        if _depsChanged:
+            self.dataDependentsChanged.emit(ddeps)
 
-            if _typeChanged:
-                self.dataTypeChanged.emit(dtype)
+        if _fieldsChanged:
+            self.dataFieldsChanged.emit(daxes + ddeps)
 
-            if _structChanged:
-                self.dataStructureChanged.emit(self.dataStructure)
-                self.newDataStructure.emit(
-                    self.dataStructure, self.dataShapes, self.dataType)
+        if _typeChanged:
+            self.dataTypeChanged.emit(dtype)
 
-            if _shapesChanged and not _structChanged:
-                self.dataShapesChanged.emit(dshapes)
+        if _structChanged:
+            self.dataStructureChanged.emit(self.dataStructure)
+            self.newDataStructure.emit(
+                self.dataStructure, self.dataShapes, self.dataType)
 
-        else:
-            dtype = type(dataIn)
-            daxes = None
-            ddeps = None
-            dshapes = None
-
-            if dtype != self.dataType:
-                _typeChanged = True
-            if _typeChanged:
-                self.dataTypeChanged.emit(dtype)
+        if _shapesChanged and not _structChanged:
+            self.dataShapesChanged.emit(dshapes)
 
         if not self.validateOptions(dataIn):
             self.logger().debug("Option validation not passed")
@@ -309,8 +332,9 @@ class Node(NodeBase):
 
         return dict(dataOut=dataIn)
 
+EmbedWidgetType = TypeVar("EmbedWidgetType", bound=QtWidgets.QWidget)
 
-class NodeWidget(QtWidgets.QWidget):
+class NodeWidget(QtWidgets.QWidget, Generic[EmbedWidgetType]):
     """
     Base class for Node control widgets.
 
@@ -320,7 +344,7 @@ class NodeWidget(QtWidgets.QWidget):
     """
 
     #: icon for this node
-    icon = None
+    icon: Optional[QtGui.QIcon] = None
 
     #: preferred location of the widget when used as dock widget
     preferredDockWidgetArea = QtCore.Qt.LeftDockWidgetArea
@@ -332,18 +356,18 @@ class NodeWidget(QtWidgets.QWidget):
     #: signal (args: (object)) all options to the node.
     allOptionsToNode = Signal(object)
 
-    def __init__(self, parent: QtWidgets.QWidget = None,
-                 embedWidgetClass: Type[QtWidgets.QWidget] = None,
-                 node: Node = None):
+    def __init__(self, parent: Optional[QtWidgets.QWidget] = None,
+                 embedWidgetClass: Optional[Type[EmbedWidgetType]] = None,
+                 node: Optional[Node] = None):
         super().__init__(parent)
 
-        self.optGetters: Dict[str, Any] = {}
-        self.optSetters: Dict[str, Any] = {}
+        self.optGetters: Dict[str, Callable[[], Any]] = {}
+        self.optSetters: Dict[str, Callable[[Any], None]] = {}
         self.node = node
 
         self._emitGuiChange = True
 
-        self.widget: Optional[QtWidgets.QWidget] = None
+        self.widget: Optional[EmbedWidgetType] = None
 
         if embedWidgetClass is not None:
             layout = QtWidgets.QVBoxLayout()
@@ -361,7 +385,7 @@ class NodeWidget(QtWidgets.QWidget):
         return ret
 
     @updateGuiFromNode
-    def setOptionFromNode(self, opt: str, value: Any):
+    def setOptionFromNode(self, opt: str, value: Any) -> None:
         """Set an option from the node
 
         Calls the set function specified in the class' ``optSetters``.
@@ -373,7 +397,7 @@ class NodeWidget(QtWidgets.QWidget):
         self.optSetters[opt](value)
 
     @Slot(dict)
-    def setOptionsFromNode(self, opts: Dict[str, Any]):
+    def setOptionsFromNode(self, opts: Dict[str, Any]) -> None:
         """Set all options without triggering updates back to the node."""
         for opt, val in opts.items():
             self.setOptionFromNode(opt, val)
